@@ -55,8 +55,15 @@ sub validate {
     }
   };
 
-  die "Parameter '$name': $@"
-    if $@;
+  if ($@) {
+    if (blessed $@ && $@->isa ('MooseX::Method::Exception')) {
+      $@->error ("Parameter ($name): " . $@->error);
+
+      $@->rethrow;
+    } else {
+      die $@;
+    }
+  }
 
   return $args;
 }
@@ -72,6 +79,8 @@ sub export {
   return $export;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
